﻿using EPiServer;
using EPiServer.Commerce.Catalog.ContentTypes;
using EPiServer.Commerce.Marketing;
using EPiServer.ServiceLocation;
using Mediachase.Commerce.Catalog;
using Mediachase.Commerce.Orders;
using Mediachase.Commerce.WorkflowCompatibility;
using System.Collections.Generic;
using System.Linq;

namespace Mediachase.Commerce.Workflow.Activities
{
    /// <summary>
    /// NOTE: This is a pre-release API that is UNSTABLE and might not satisfy the compatibility requirements as denoted by its associated normal version.
    /// Calculate discounts using the new promotion engine
    /// </summary>
    public class CalculateDiscountsVNextActivity : OrderGroupActivityBase
    {
        /// <summary>
        /// Executes the specified execution context.
        /// </summary>
        /// <param name="executionContext">The execution context.</param>
        /// <returns></returns>
        protected override ActivityExecutionStatus Execute(ActivityExecutionContext executionContext)
        {
            ValidateRuntime();
            ServiceLocator.Current.GetInstance<IPromotionEngine>().Run(OrderGroup);
            var referenceConverter = ServiceLocator.Current.GetInstance<ReferenceConverter>();
            var contentLoader = ServiceLocator.Current.GetInstance<IContentLoader>();
            var order = OrderGroup as IOrderGroup;
            var entryCodes = GetAffectedEntryCodes(order);
            var codeContentLinkMap = referenceConverter.GetContentLinks(entryCodes);
            foreach (var contentLink in codeContentLinkMap.Values)
            {
                var item = contentLoader.Get<EntryContentBase>(contentLink);
                if (!(item is IPricing))
                {
                    continue;
                }

                foreach (var lineItem in order.Forms.SelectMany(x => x.Shipments).SelectMany(x => x.LineItems).Where(x => x.Code.Equals(item.Code)))
                {
                    var metaItem = OrderGroup.OrderForms.SelectMany(x => x.LineItems).FirstOrDefault(x => x.LineItemId == lineItem.LineItemId);
                    if (metaItem != null)
                    {
                        metaItem.LineItemDiscountAmount = lineItem.LineItemDiscountAmount;
                        metaItem.OrderLevelDiscountAmount = lineItem.OrderLevelDiscountAmount;
                    }
                }
            }

            return ActivityExecutionStatus.Closed;
        }

        private static IEnumerable<string> GetAffectedEntryCodes(IOrderGroup order)
        {
            if (HasOrderPromotionWithSavedAmount(order))
            {
                return GetAllEntryCodesFromOrder(order);
            }

            return GetAffectedEntryCodesFromEntryPromotions(order);
        }

        private static bool HasOrderPromotionWithSavedAmount(IOrderGroup order)
        {
            return order.Forms.SelectMany(f => f.Promotions).Any(x => x.OrderForm.SavedAmount > 0);
        }

        private static IEnumerable<string> GetAffectedEntryCodesFromEntryPromotions(IOrderGroup order)
        {
            return order.Forms.SelectMany(f => f.Promotions).SelectMany(p => p.Entries).Select(i => i.EntryCode);
        }

        private static IEnumerable<string> GetAllEntryCodesFromOrder(IOrderGroup order)
        {
            return order.Forms.SelectMany(x => x.Shipments).SelectMany(x => x.LineItems).Select(x => x.Code);
        }
    }
}
